@@ -23,9 +23,6 @@
 #include "config.h"
 
 #include "gkr-daemon.h"
-#include "gkr-daemon-util.h"
-
-#include "common/gkr-location.h"
 
 #include "egg/egg-buffer.h"
 #include "egg/egg-secure-memory.h"
@@ -42,6 +39,9 @@
 #include "ui/gkr-ask-request.h"
 #include "ui/gkr-ask-daemon.h"
 
+#include "util/gkr-daemon-util.h"
+#include "util/gkr-location.h"
+
 #include <unistd.h>
 #include <errno.h>
 #include <stdlib.h>
