@@ -24,13 +24,11 @@
 
 #include "gkr-daemon.h"
 
-#include "common/gkr-async.h"
-#include "common/gkr-cleanup.h"
-#include "common/gkr-crypto.h"
-#include "common/gkr-daemon-util.h"
+#include "egg/egg-cleanup.h"
+#include "egg/egg-libgcrypt.h"
 #include "egg/egg-secure-memory.h"
 #include "egg/egg-unix-credentials.h"
-#include "common/gkr-unix-signal.h"
+#include "egg/egg-unix-signal.h"
 
 #include "keyrings/gkr-keyring-login.h"
 
@@ -40,6 +38,9 @@
 
 #include "ui/gkr-ask-daemon.h"
 
+#include "util/gkr-daemon-async.h"
+#include "util/gkr-daemon-util.h"
+
 #include <unistd.h>
 #include <fcntl.h>
 #include <errno.h>
@@ -124,7 +125,7 @@ parse_arguments (int *argc, char** argv[])
 	/* Take ownership of the string */
 	if (run_components) {
 		run_components = g_strdup (run_components);
-		gkr_cleanup_register (g_free, run_components);
+		egg_cleanup_register (g_free, run_components);
 	}
 	
 	/* Check the arguments */
@@ -232,7 +233,7 @@ egg_memory_unlock (void)
 }
 
 void*
-egg_memory_fallback (void *p, unsigned long sz)
+egg_memory_fallback (void *p, size_t sz)
 {
 	const gchar *env;
 	
@@ -401,7 +402,7 @@ read_login_password (int fd)
 static void
 cleanup_and_exit (int code)
 {
-	gkr_cleanup_perform ();
+	egg_cleanup_perform ();
 	exit (code);
 }
 
@@ -418,7 +419,7 @@ lifetime_slave_pipe_io (GIOChannel  *channel,
 			GIOCondition cond,
 			gpointer     callback_data)
 {
-	gkr_cleanup_perform ();
+	egg_cleanup_perform ();
 	_exit (2);
 	return FALSE;
 }
@@ -464,7 +465,7 @@ initialize_other_running_daemon (int sock)
 	if (egg_unix_credentials_write (sock) < 0)
 		return FALSE;
 
-	egg_buffer_init_full (&buf, 128, (EggBufferAllocator)g_realloc);
+	egg_buffer_init_full (&buf, 128, g_realloc);
 	
 	envp = gnome_keyring_build_environment (GNOME_KEYRING_IN_ENVIRONMENT);
 	ret = gkr_proto_encode_prepare_environment (&buf, (const gchar**)envp);
@@ -578,31 +579,32 @@ fork_and_print_environment (void)
 		exit (0);
 	}
 	
-	if (!run_daemonized) 
-		return;
+	if (run_daemonized) { 
+		
+		/* Double fork if need to daemonize properly */
+		pid = fork ();
 	
-	/* Double fork if need to daemonize properly */
-	pid = fork ();
+		if (pid != 0) {
+
+			/* Here we are in the intermediate child process */
+				
+			/* 
+			 * This process exits, so that the final child will inherit 
+			 * init as parent to avoid zombies
+			 */
+			if (pid == -1)
+				exit (1);
 	
-	if (pid != 0) {
-
-		/* Here we are in the intermediate child process */
-			
-		/* 
-		 * This process exits, so that the final child will inherit 
-		 * init as parent to avoid zombies
-		 */
-		if (pid == -1)
-			exit (1);
-
-		/* We've done two forks. Now we know the PID */
-		print_environment (pid);
-			
-		/* The intermediate child exits */
-		exit (0);
+			/* We've done two forks. Now we know the PID */
+			print_environment (pid);
+				
+			/* The intermediate child exits */
+			exit (0);
+		}
+		
 	}
 
-	/* Here we are in the resulting daemon process. */
+	/* Here we are in the resulting daemon or background process. */
 
 	for (i = 0; i < 3; ++i) {
 		fd = open ("/dev/null", O_RDONLY);
@@ -611,19 +613,9 @@ fork_and_print_environment (void)
 	}
 }
 
-gboolean
-gkr_daemon_complete_initialization(void)
+static gboolean
+gkr_daemon_complete_initialization_steps (void)
 {
-	/*
-	 * Sometimes we don't initialize the full daemon right on 
-	 * startup. When run with --login is one such case.
-	 */
-	
-	if (initialization_completed) {
-		g_message ("The daemon was already initialized.");
-		return TRUE;
-	}
-	
 	/* Initialize new style PKCS#11 components */
 	if (!gkr_pkcs11_daemon_initialize ())
 		return FALSE;
@@ -648,6 +640,28 @@ gkr_daemon_complete_initialization(void)
 	return TRUE;
 }
 
+gboolean
+gkr_daemon_complete_initialization (void)
+{
+	/*
+	 * Sometimes we don't initialize the full daemon right on 
+	 * startup. When run with --login is one such case.
+	 */
+	
+	if (initialization_completed) {
+		g_message ("The daemon was already initialized.");
+		return TRUE;
+	}
+
+	/* Set this early so that two initializations don't overlap */
+	initialization_completed = TRUE;
+
+	/* But then set it back if initializing fails */
+	initialization_completed = gkr_daemon_complete_initialization_steps ();
+
+	return initialization_completed;
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -687,8 +701,8 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 #endif
 
-	gkr_crypto_setup ();
-
+	egg_libgcrypt_initialize ();
+	
 	/* Send all warning or error messages to syslog */
 	prepare_logging ();
 	
@@ -703,7 +717,7 @@ main (int argc, char *argv[])
 	/* Initialize our daemon main loop and threading */
 	loop = g_main_loop_new (NULL, FALSE);
 	ctx = g_main_loop_get_context (loop);
-	gkr_async_workers_init (loop);
+	gkr_daemon_async_workers_init (loop);
 	
 	/* 
 	 * Always initialize the keyring subsystem. This is a necessary
@@ -731,9 +745,9 @@ main (int argc, char *argv[])
 	prepare_logging();
 
 	signal (SIGPIPE, SIG_IGN);
-	gkr_unix_signal_connect (ctx, SIGINT, signal_handler, NULL);
-	gkr_unix_signal_connect (ctx, SIGHUP, signal_handler, NULL);
-	gkr_unix_signal_connect (ctx, SIGTERM, signal_handler, NULL);
+	egg_unix_signal_connect (ctx, SIGINT, signal_handler, NULL);
+	egg_unix_signal_connect (ctx, SIGHUP, signal_handler, NULL);
+	egg_unix_signal_connect (ctx, SIGTERM, signal_handler, NULL);
              
 	/* TODO: Do we still need this? XFCE still seems to use it. */
 	slave_lifetime_to_fd ();
@@ -751,13 +765,13 @@ main (int argc, char *argv[])
 	g_main_loop_run (loop);
 
 	/* Make sure no other threads are running */
-	gkr_async_workers_stop_all ();
+	gkr_daemon_async_workers_stop_all ();
 	
 	/* This wraps everything up in order */
-	gkr_cleanup_perform ();
+	egg_cleanup_perform ();
 	
 	/* Final shutdown of anything workers running about */
-	gkr_async_workers_uninit ();
+	gkr_daemon_async_workers_uninit ();
 
 	return 0;
 }
